#ifndef LOGENTRY_HPP
#define LOGENTRY_HPP

#include "EntryType.hpp"
#include "ExtendedLog_global.h"

#include <QDateTime>
#include <QObject>
#include <QString>

namespace ExtendedLog
{
	/**
	 * @brief A single log-entry
	 */
	class EXTENDEDLOG_EXPORT LogEntry: public QObject
	{
			Q_OBJECT

		public:
			/**
			 * @brief Copy-ctor
			 * @param other The entry to copy
			 */
			LogEntry(const LogEntry &other);

			/**
			 * @brief Create a new log-entry with specified type and message
			 * @param type The EntryType of this entry
			 * @param message The message of this entry
			 */
			LogEntry(EntryType type, QString message);

			/**
			 * @brief Create a new log-entry with specified type, message and timestamp
			 * @param dateTime The timestamp to use for this entry
			 * @param type The EntryType of this entry
			 * @param message The message of this entry
			 */
			LogEntry(QString dateTime, QString type, QString message);

			/**
			 * @brief Assignment operator for const-references
			 * @param other The other entry
			 * @return This entry with the values of \p other
			 */
			LogEntry& operator=(const LogEntry &other);

			QString toString();

			/**
			 * @brief Get the type of this entry
			 * @return The type of this entry as EntryType
			 */
			EntryType getEntryType();

			/**
			 * @brief Get the type of this entry as String
			 * @return The type of this entry as String
			 */
			QString typeString();

			/**
			 * @brief Get date and time of this entry
			 * @return The timestamp of this entry
			 */
			QDateTime& getDateTime();

			/**
			 * @brief Get the entry-message
			 * @return The message of this entry
			 */
			QString& getMessage();

			/**
			 * @brief Set the type of this entry to \p value
			 * @param value the EntryType as string
			 */
			void setEntryType(QString value);

			/**
			 * @brief Try to set the timestamp of this entry
			 * @param value The timestamp as string in format dd.MM.yyyy hh:mm:ss
			 * @return <tt>true</tt> if \p value is a valid timestamp in the correct format, <tt>false</tt> otherwise
			 */
			bool trySetDateTime(QString value);

		private:
			/**
			 * @brief the type of this entry
			 */
			EntryType type;

			/**
			 * @brief The creation timestamp of this entry
			 */
			QDateTime dateTime;

			/**
			 * @brief the log-message itself
			 */
			QString message;

			/**
			 * @brief Get an entry-type from its string representation
			 * @param typeString The type as string
			 * @return The type as EntryType
			 */
			EntryType getEntryType(QString typeString);

			/**
			 * @brief Generate string-representation of this entry
			 * @return The EntryType of this entry as string
			 */
			QString entryTypeToString();
	};
} // namespace ExtendedLog

#endif // LOGENTRY_HPP
